-- THIS FILE IS PART OF PROJECT WORKBENCH
-- ai_control.Lua - An example of ai control in Lumberyard
-- utilities.lua and StateMachine.lua are needed

-- Copyright (c) 2018 Project Workbench

local utilities = require "scripts/common/utilities"
local StateMachine = require "scripts/common/StateMachine"

local aicontroller = 
{
    Properties = 
    {

		playerEntity = { default = EntityId(), description = "the player character" },	
		
		-- the threshold of becoming suspicious
		distanceThreshold = 4.0, 
	
		
		StateMachines =
		{
			AI =
			{
				InitialState = "Idle",
				DebugStateMachine = false,
				
				Idle = {},
				Suspicious = {},			
			}		
		}		
	},
	
	-- to record the current distance between enemy and player
	distanceLength = 0.0, 
	
	
	
	AIStates =
	{
		Idle =
		{
			OnEnter = function(self, sm)
				Debug.Log("Enter Idle state");		
			end,
			
			OnExit = function(self, sm)
				
			end,
			
			OnUpdate = function(self, sm, deltaTime)
				
			end,

			Transitions =
			{
				-- Become suspicious if the player is nearby.
				Suspicious =
				{
					Evaluate = function(state, sm)
						return sm.UserData:checkSuspicious();
					end
				},

				
			},
		},
		
		Suspicious =
		{
			OnEnter = function(self, sm)
				Debug.Log("Enter Suspicious State");
				
				--grow higher when entering the suspicious state
				sm.UserData:growUp();
				
			end,
			
			OnExit = function(self, sm)
				
				--back to normal when exit the suspicious state
				sm.UserData:shrinkDown();
			end,
			
			OnUpdate = function(self, sm, deltaTime)

			end,

			Transitions =
			{
				-- Become idle if the player is not nearby
				Idle =
				{
					Evaluate = function(state, sm)
						return sm.UserData:checkIdle();
					end
				},			
			},
		},		
	},	
}




function aicontroller:OnActivate()

	
	self.AIStateMachine = {}
	setmetatable(self.AIStateMachine, StateMachine);
	self.AIStateMachine:Start("AI", self.entityId, self, self.AIStates, true, self.Properties.StateMachines.AI.InitialState, self.Properties.StateMachines.AI.DebugStateMachine);

	self.tickBusHandler = TickBus.Connect(self);
	
end


function aicontroller:OnTick(deltaTime, timePoint)


	local playerTm = TransformBus.Event.GetWorldTM(self.Properties.playerEntity);
	local playerTranslation = playerTm:GetTranslation();
	
	
	local myTm = TransformBus.Event.GetWorldTM(self.entityId);
	local myTranslation = myTm:GetTranslation();
	

	local distanceVector = playerTranslation - myTranslation;
	self.distanceLength = distanceVector:GetLength();

	
end

function aicontroller:growUp()

	TransformBus.Event.SetLocalScaleZ(self.entityId, 5.0);
	
end


function aicontroller:shrinkDown()

	TransformBus.Event.SetLocalScaleZ(self.entityId, 1.0);
	
end


function aicontroller:checkSuspicious()
	
	
	if (self.distanceLength < self.Properties.distanceThreshold) then
		return true;	
	else
		return false;
	end
	
end


function aicontroller:checkIdle()
	
	
	if (self.distanceLength > self.Properties.distanceThreshold) then
		return true;	
	else
		return false;
	end
	
end

return aicontroller;